<?php
/* Class: Artist */

class Artist {
    const EARLIEST_DATE_ALLOWED = 'January 1, 1200';

    private static $artistCount = 0;

    private $firstName;
    private $lastName;
    private $birthCity;
    private $birthDate;
    private $deathDate;

    function __construct($firstName, $lastName, $birthCity, $birthDate, $deathDate = null) {
        $this->setFirstName($firstName);
        $this->setLastName($lastName);
        $this->setBirthCity($birthCity);
        $this->setBirthDate($birthDate);
        $this->setDeathDate($deathDate);

        self::$artistCount++;
    }

    public static function getArtistCount()  { return self::$artistCount;                       }
    public function getFirstName()           { return $this->firstName;                         }
    public function getLastName()            { return $this->lastName;                          }
    public function getName()                { return $this->firstName . ' ' . $this->lastName; }
    public function getBirthCity()           { return $this->birthCity;                         }
    public function getBirthDate()           { return $this->birthDate;                         }
    public function getDeathDate()           { return $this->deathDate;                         }
    public function getEarliestDateAllowed() { return date_create(self::EARLIEST_DATE_ALLOWED); }

    public function setFirstName($firstName) { $this->firstName = $firstName; }
    public function setLastName($lastName)   { $this->lastName = $lastName;   }
    public function setBirthCity($birthCity) { $this->birthCity = $birthCity; }
    public function setBirthDate($birthDate) {
        /* set to null if invalid */
        $date = date_create($birthDate);
        $earliestAllowedDate = $this->getEarliestDateAllowed();

        if (!$date || $date < $earliestAllowedDate) {
            /* if null or out of range then assign null. */
            $date = null;
        }
        $this->birthDate = $date;
    }
    public function setDeathDate($deathDate) {
        $date = date_create($deathDate);
        $birthDate = date_create( $this->getBirthDate() );

        if ( !$date || $date <  $birthDate) {
            /* if null or out of range then assign null */
            $this->deathDate = null;
        } else {
            $this->deathDate = $date;
        }
    }

    public function __toString() {
        $bday = date_format($this->getBirthDate(), 'd M Y');
        $dday = date_format($this->getDeathDate(), 'd M Y');

        $table  = '<table>
                    <tr><th colspan="2">' . $this->getName() . '</th></tr>
                    <tr><td>Birth:</td><td>' . $bday . ' (' . $this->getBirthCity() . ')</td></tr>
                    <tr><td>Death:</td><td>' . $dday . '</td></tr>
                   </table>';

        return $table;
    }
}// end Artist